/******************************************************************

    CBBFRLS.C

    Ismo Krkkinen

    Brute force using randomized local search for a range of
    number of clusters.

******************************************************************/

#define ProgName       "CBBFRLS"
#define VersionNumber  "Version 0.2.6"
#define LastUpdated    "9.10.2002" /* iak */

#define  FACTFILE  "cbbfrls.fac"

#include "parametr.c"
#include "rls.h"
#include "cb.h"
#include "file.h"
#include "interfc.h"
#include "memctrl.h"
#include "random.h"
#include "cb_util.h"
#include "DistCrit/distance.h"
#include "DistCrit/criteria.h"
#include "iter.h"
#include "sortcb.h"
#include "brute_rls.h"

#define MAXFILENAME 512

#ifndef max
#define max(a,b) ((a) > (b) ? (a) : (b))
#endif

void PrintInfo() {
    PrintMessage("%s\t%s\n(last modified %s, compiled %s, %s)\n\n"
        "Brute force using RLS for a range of cluster numbers\n"
        "Usage: %s data.ts out.cb [out.pa] %coptions\n\n",
        ProgName, VersionNumber, LastUpdated, __DATE__, __TIME__,
	ProgName, OPTION_SYMBOL);
    PrintOptions();
    PrintMessage("\n");
}

void PrintOperatingInfo() {
    if (Value(QuietLevel)) {
        PrintMessage("\n%s %s %s\n\n", ProgName, VersionNumber, LastUpdated);
        PrintSelectedOptions();
    }
}

static char DataName[MAXFILENAME] = "\0";
static char OutCBName[MAXFILENAME] = "\0";
static char OutPAName[MAXFILENAME] = "\0";

#define PARAMETER_FILENAME_COUNT 3

static ParameterInfo paraminfo[PARAMETER_FILENAME_COUNT] = { 
    { DataName, "", 0, INFILE },
    { OutCBName, "", 0, OUTFILE },
    { OutPAName, "", 1, OUTFILE } };

void CheckParameters() {
    if (Value(Distance) == EvaluationFunctionDefault) SetValue(Distance,
	ciCriterionDefaultDistance(Value(EvaluationFunction)));
    if ((Value(HaltingCriterion) == TTimes) && Value(HaltingLimit) < 1001) {
	ErrorMessage(
	    "Multiplier for T-times should be at least 1001 (1.001).\n");
	ExitProcessing(1);
    }
    if ((Value(SwapMethod) == NoSwap) && !Value(GLAIterations)) {
	ErrorMessage("No swap and no GLA-iterations: nothing happens.\n");
	ExitProcessing(2);
    }
    if (ExistFile(OutCBName) && !Value(Overwrite)) {
	ErrorMessage("Output codebook exists and no overwrite specified.\n");
	ExitProcessing(3);
    }
    if (ExistFile(OutPAName) && !Value(Overwrite)) {
	ErrorMessage("Output partition exists and no overwrite specified.\n");
	ExitProcessing(4);
    }
    if (!*DataName || !ExistFile(DataName)) {
	ErrorMessage("Input data file not found.\n");
	ExitProcessing(7);
    }
    if (Value(MaxNumberOfClusters) < Value(MinNumberOfClusters)) {
	ErrorMessage("Bad range: %i < %i.\n",
	    Value(MaxNumberOfClusters), Value(MinNumberOfClusters));
	ExitProcessing(8);
    }
    if (Value(HaltingCriterion) == TTimes) SetValue(HaltingLimit,
	(int)(1000000 / (Value(HaltingLimit) / 1000.0)));
}

static float MinimumValue = 0;
static float MaximumValue = 0;
static int FileFormat = 0;

static void Initialize(TRAININGSET* TS) {
    if (ReadGenericData(DataName, TS,
	&FileFormat, &MinimumValue, &MaximumValue))
    {
	ErrorMessage("Failed to read data.\n");
	ExitProcessing(11);
    }
    if (FileFormat != RGD_CB && !*OutPAName) {
	if (Value(QuietLevel) > 1) PrintMessage(
	    "Data vectors before duplicate removal: %i,", BookSize(TS));
	RemoveDuplicatesFromCodebook(TS);
	if (Value(QuietLevel) > 1) PrintMessage(" after: %i.\n", BookSize(TS));
    }
    if (BookSize(TS) < Value(MaxNumberOfClusters)) {
	ErrorMessage("More max clusters than data vectors (%i < %i).\n",
	    BookSize(TS), Value(MaxNumberOfClusters));
	ExitProcessing(10);
    }
}

static void Finalize(TRAININGSET* TS, CODEBOOK* CB, PARTITIONING* PA) {
    if (!*OutPAName) SortCodebook(CB, DATA_ASCENDING);
    else WriteGenericMapping(OutPAName, TS, PA, FileFormat, Value(Overwrite));
    WriteGenericData(OutCBName, CB, 
	FileFormat, MinimumValue, MaximumValue, Value(Overwrite));
    FreeCodebook(TS);
    FreeCodebook(CB);
    FreePartitioning(PA);
}

int main(int argc, char** argv) {
    TRAININGSET TS;
    CODEBOOK CB;
    PARTITIONING PA;
    CriterionInfo* CI;
    DistanceInfo* DI;
    Iterations* Iter;
    int ReturnValue;
    float Error;

    ParseParameters(argc, argv, PARAMETER_FILENAME_COUNT, paraminfo);
    initrandom(Value(RandomNumberSeed));
    CheckParameters();
    PrintOperatingInfo();
    Initialize(&TS);
    DI = diNew(&TS, Value(Distance), 0, 0, 0, 0);
    CI = ciNew(&TS, DI, Value(EvaluationFunction), 0, 0, 0, 0, 0, 0);
    Iter = itNew(Value(HaltingCriterion) + 1, Value(HaltingIterations),
	Value(HaltingLimit) / 1000000.0f, 100 * Value(HaltingIterations), 0);

    ReturnValue = BruteForce(&TS, &CB, &PA,
	Value(MinNumberOfClusters), Value(MaxNumberOfClusters), CI,
	&Error, Iter, Value(GLAIterations), Value(SwapMethod), 
	max(0, Value(QuietLevel) - 1), Value(TraceData), Value(TraceDataID));

    if (Value(QuietLevel) == 1) PrintMessage(
	"Best: %i groups, error is %i#.6g.\n", BookSize(&CB), Error);
    itDelete(Iter);
    ciDelete(CI);
    diDelete(DI);
    Finalize(&TS, &CB, &PA);
    return ReturnValue;
}

